/******************************************************************************
 *
 * Copyright (C) 1996-2000 Jason Evans <jasone@canonware.com>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice(s), this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified other than the possible
 *    addition of one or more copyright notices.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice(s), this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 *
 * Version: 1.0.0
 *
 ******************************************************************************/

/* Pseudo-opaque type. */
typedef struct cw_dch_s cw_dch_t;

struct cw_dch_s {
#if (defined(_LIBSTASH_DBG) || defined(_LIBSTASH_DEBUG))
	cw_uint32_t	magic;

	/* Counters used to get an idea of performance. */
	cw_uint32_t	num_grows;
	cw_uint32_t	num_shrinks;
#endif

	/* Allocator. */
	cw_mem_t	*mem;

	cw_bool_t	is_malloced;

	/*
	 * Intial table size and high/low water marks.  These values are
	 * used in proportion if the table grows.
	 */
	cw_uint32_t	base_table;
	cw_uint32_t	base_grow;
	cw_uint32_t	base_shrink;

	/* (grow_factor * base_table) is the current table size. */
	cw_uint32_t	grow_factor;

	/* Cached for later ch creation during rehashes. */
	cw_uint32_t	(*hash)(const void *);
	cw_bool_t	(*key_comp)(const void *, const void *);

	/* Where all of the real work is done. */
	cw_ch_t		*ch;
};

cw_dch_t	*dch_new(cw_dch_t *a_dch, cw_mem_t *a_mem, cw_uint32_t
    a_base_table, cw_uint32_t a_base_grow, cw_uint32_t a_base_shrink,
    cw_ch_hash_t *a_hash, cw_ch_key_comp_t *a_key_comp);
void		dch_delete(cw_dch_t *a_dch);

cw_uint32_t	dch_count(cw_dch_t *a_dch);

cw_bool_t	dch_insert(cw_dch_t *a_dch, const void *a_key, const void
    *a_data, cw_chi_t *a_chi);
cw_bool_t	dch_remove(cw_dch_t *a_dch, const void *a_search_key, void
    **r_key, void **r_data, cw_chi_t **r_chi);
cw_bool_t	dch_search(cw_dch_t *a_dch, const void *a_key, void **r_data);

cw_bool_t	dch_get_iterate(cw_dch_t *a_dch, void **r_key, void **r_data);
cw_bool_t	dch_remove_iterate(cw_dch_t *a_dch, void **r_key, void
    **r_data, cw_chi_t **r_chi);

void		dch_dump(cw_dch_t *a_dch, const char *a_prefix);
