/******************************************************************************
 *
 * Copyright (C) 1996-2000 Jason Evans <jasone@canonware.com>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice(s), this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified other than the possible
 *    addition of one or more copyright notices.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice(s), this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 *
 * Version: 1.0.0
 *
 ******************************************************************************/

typedef cw_bool_t cw_mem_oom_handler_t(const void *a_data, cw_uint32_t a_size);

struct cw_mem_s {
	cw_mem_t	*mem;
	cw_bool_t	is_malloced;

	cw_mtx_t	lock;

#if (defined(_LIBSTASH_DBG) || defined(_LIBSTASH_DEBUG))
#define _CW_MEM_TABLE_SIZE 1024		/* Number of slots in hash table. */
	cw_ch_t		*addr_hash;
#endif

	cw_mem_oom_handler_t *oom_handler;
        const void	*handler_data;
};

cw_mem_t *mem_new(cw_mem_t *a_mem, cw_mem_t *a_internal);

void	mem_delete(cw_mem_t *a_mem);

void	mem_set_oom_handler(cw_mem_t *a_mem, cw_mem_oom_handler_t
    *a_oom_handler, const void *a_data);

void	*mem_malloc(cw_mem_t *a_mem, size_t a_size, const char *a_filename,
    cw_uint32_t a_line_num);

void	*mem_calloc(cw_mem_t *a_mem, size_t a_number, size_t a_size, const
    char *a_filename, cw_uint32_t a_line_num);

void	*mem_realloc(cw_mem_t *a_mem, void *a_ptr, size_t a_size, const char
    *a_filename, cw_uint32_t a_line_num);

void	mem_free(cw_mem_t *a_mem, void *a_ptr, const char *a_filename,
    cw_uint32_t a_line_num);

/*
 * These macros are declared differently, depending on whether this is a debug
 * library, because consistently using arguments of NULL and 0 reduces the size
 * of the generated binary.  Since these arguments aren't used in the optimized
 * library anyway, this is a free (though perhaps small) memory savings.
 */
#if (defined(_LIBSTASH_DBG) || defined(_LIBSTASH_DEBUG))
#define _cw_mem_malloc(a_mem, a_size)					\
	mem_malloc((a_mem), (a_size), __FILE__, __LINE__)
#define _cw_mem_calloc(a_mem, a_number, a_size)				\
	mem_calloc((a_mem), (a_number), (a_size), __FILE__, __LINE__)
#define _cw_mem_realloc(a_mem, a_ptr, a_size)				\
	mem_realloc((a_mem), (a_ptr), (a_size), __FILE__, __LINE__)
#define	_cw_mem_free(a_mem, a_ptr)					\
	mem_free((a_mem), (a_ptr), __FILE__, __LINE__)

#define _cw_malloc(a_size)						\
	mem_malloc(cw_g_mem, (a_size), __FILE__, __LINE__)
#define _cw_calloc(a_number, a_size)					\
	mem_calloc(cw_g_mem, (a_number), (a_size), __FILE__, __LINE__)
#define _cw_realloc(a_ptr, a_size)					\
	mem_realloc(cw_g_mem, (a_ptr), (a_size), __FILE__, __LINE__)
#define	_cw_free(a_ptr)							\
	mem_free(cw_g_mem, (a_ptr), __FILE__, __LINE__)

#else
#define _cw_mem_malloc(a_mem, a_size)					\
	mem_malloc((a_mem), (a_size), NULL, 0)
#define _cw_mem_calloc(a_mem, a_number, a_size)				\
	mem_calloc((a_mem), (a_number), (a_size), NULL, 0)
#define _cw_mem_realloc(a_mem, a_ptr, a_size)				\
	mem_realloc((a_mem), (a_ptr), (a_size), NULL, 0)
#define	_cw_mem_free(a_mem, a_ptr)					\
	mem_free((a_mem), (a_ptr), NULL, 0)

#define _cw_malloc(a_size)						\
	mem_malloc(cw_g_mem, (a_size), NULL, 0)
#define _cw_calloc(a_number, a_size)					\
	mem_calloc(cw_g_mem, (a_number), (a_size), NULL, 0)
#define _cw_realloc(a_ptr, a_size)					\
	mem_realloc(cw_g_mem, (a_ptr), (a_size), NULL, 0)
#define	_cw_free(a_ptr)							\
	mem_free(cw_g_mem, (a_ptr), NULL, 0)

#endif
