/******************************************************************************
 *
 * Copyright (C) 1996-2000 Jason Evans <jasone@canonware.com>.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice(s), this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified other than the possible
 *    addition of one or more copyright notices.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice(s), this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 *
 * Version: 1.0.0
 *
 ******************************************************************************/

/*
 * Typedef's to allow easy function pointer passing.
 */
typedef cw_uint32_t cw_out_render_t(const char *a_format, cw_uint32_t
    a_format_len, const void *a_arg, cw_uint32_t a_max_len, cw_uint8_t *r_str);

/* Maximum type string length, including NULL termination. */
#define _CW_OUT_MAX_TYPE 16

/* Pseudo-opaque types. */
typedef struct cw_out_s cw_out_t;
typedef struct cw_out_ent_s cw_out_ent_t;

struct cw_out_s {
#if (defined(_LIBSTASH_DBG) || defined(_LIBSTASH_DEBUG))
	cw_uint32_t	magic;
#endif

	/* Allocator. */
	cw_mem_t	*mem;

	cw_bool_t	is_malloced;
	cw_sint32_t	fd;

	cw_mtx_t	lock;

	cw_uint32_t	nextensions;
	cw_out_ent_t	*extensions;
};

struct cw_out_ent_s {
	cw_uint8_t	type[_CW_OUT_MAX_TYPE];
	cw_uint32_t	len;
	cw_uint32_t	size;
	cw_out_render_t	*render_func;
};

cw_out_t	*out_new(cw_out_t *a_out, cw_mem_t *a_mem);

void		out_delete(cw_out_t *a_out);

cw_bool_t	out_register(cw_out_t *a_out, const char *a_type, cw_uint32_t
    a_size, cw_out_render_t * a_render_func);

cw_bool_t	out_merge(cw_out_t *a_a, cw_out_t *a_b);

cw_sint32_t	out_get_default_fd(cw_out_t *a_out);

void		out_set_default_fd(cw_out_t *a_out, cw_sint32_t a_fd);

#define		_cw_out_put(a_format, a_args...)			\
	out_put(cw_g_out, a_format , ## a_args)
cw_sint32_t	out_put(cw_out_t *a_out, const char *a_format,...);

#define		_cw_out_put_e(a_format, a_args...)			\
	out_put_e(cw_g_out, __FILE__, __LINE__, __FUNCTION__, a_format	\
	    , ## a_args)
cw_sint32_t	out_put_e(cw_out_t *a_out, const char *a_file_name, cw_uint32_t
    a_line_num, const char *a_func_name, const char *a_format,...);

#define		_cw_out_put_l(a_format, a_args...)			\
	out_put_l(cw_g_out, a_format , ## a_args)
cw_sint32_t	out_put_l(cw_out_t *a_out, const char *a_format,...);

#define		_cw_out_put_le(a_format, a_args...)			\
	out_put_le(cw_g_out, __FILE__, __LINE__, __FUNCTION__, a_format	\
	    , ## a_args)
cw_sint32_t	out_put_le(cw_out_t *a_out, const char *a_file_name, cw_uint32_t
    a_line_num, const char *a_func_name, const char *a_format,...);

#define		_cw_out_put_n(a_size, a_format, a_args...)		\
	out_put_n(cw_g_out, a_size, a_format , ## a_args)
cw_sint32_t out_put_n(cw_out_t *a_out, cw_uint32_t a_size, const char
    *a_format,...);

#define		_cw_out_put_f(a_fd, a_format, a_args...)		\
	out_put_f(cw_g_out, a_fd, a_format, ## a_args)
cw_sint32_t	out_put_f(cw_out_t *a_out, cw_sint32_t a_fd, const char
    *a_format,...);

cw_sint32_t	out_put_fe(cw_out_t *a_out, cw_sint32_t a_fd, const char
    *a_file_name, cw_uint32_t a_line_num, const char *a_func_name, const char
    *a_format,...);

cw_sint32_t	out_put_fl(cw_out_t *a_out, cw_sint32_t a_fd, const char
    *a_format,...);

cw_sint32_t	out_put_fle(cw_out_t *a_out, cw_sint32_t a_fd, const char
    *a_file_name, cw_uint32_t a_line_num, const char *a_func_name, const char
    *a_format,...);

#define		_cw_out_put_fn(a_fd, a_size, a_format, a_args...)	\
	out_put_fn(cw_g_out, a_fd, a_size, a_format, ## a_args)
cw_sint32_t	out_put_fn(cw_out_t *a_out, cw_sint32_t a_fd, cw_uint32_t
    a_size, const char *a_format,...);

cw_sint32_t	out_put_fv(cw_out_t *a_out, cw_sint32_t a_fd, const char
    *a_format, va_list a_p);

#define		_cw_out_put_s(a_str, a_format, a_args...)		\
	out_put_s(cw_g_out, a_str, a_format, ## a_args)
cw_sint32_t	out_put_s(cw_out_t *a_out, char *a_str, const char
    *a_format,...);

cw_sint32_t	out_put_sa(cw_out_t *a_out, char **r_str, const char
    *a_format,...);

#define		_cw_out_put_sn(a_str, a_size, a_format, a_args...)	\
	out_put_sn(cw_g_out, a_str, a_size, a_format, ## a_args)
cw_sint32_t	out_put_sn(cw_out_t *a_out, char *a_str, cw_uint32_t a_size,
    const char *a_format,...);

cw_sint32_t	out_put_sv(cw_out_t *a_out, char *a_str, const char *a_format,
    va_list a_p);

cw_sint32_t	out_put_sva(cw_out_t *a_out, char **r_str, const char *a_format,
    va_list a_p);

cw_sint32_t	out_put_svn(cw_out_t *a_out, char *a_str, cw_uint32_t a_size,
    const char *a_format, va_list a_p);

cw_sint32_t	spec_get_type(const char *a_spec, cw_uint32_t a_spec_len, const
    char **r_val);

cw_sint32_t	spec_get_val(const char *a_spec, cw_uint32_t a_spec_len, const
    char *a_name, cw_uint32_t a_name_len, const cw_uint8_t **r_val);
